# -*- coding: utf-8 -*-

import configparser
import logging
import os
import textwrap

from . import pwdtools


logger = logging.getLogger(__name__)

config_filename = '/etc/wgfrontend/wgfrontend.conf'


class Configuration():
    """Class for reading/writing the configuration file"""
    _config = None

    def exists(self):
        """Checks whether the config file exists"""
        return os.path.isfile(self.filename)
  
    def read_config(self):
        """Reads the config file"""
        try:
            logger.debug('Attempting to read config file [{0}]'.format(self.filename))
            cfg = configparser.ConfigParser()
            cfg.read(self.filename)
            self._config = dict(cfg['general'])
            self._users = dict(cfg['users'])
        except Exception as e:
            logger.warning('Config file [{0}] could not be read [{1}], using defaults'.format(self.filename, str(e)))
            self._config = dict()
    
    def write_config(self, wg_configfile='', socket_host='0.0.0.0', socket_port=8080, user='', users={}):
        """Writes a new config file with the given attributes"""
        # Set default values
        if not wg_configfile.strip():
            wg_configfile = '/etc/wireguard/wg_rw.conf'
        if not socket_host.strip():
            socket_host = '0.0.0.0'
        if not str(socket_port).strip():
            socket_port = 8080
        if not user.strip():
            user = 'wgfrontend'
        users = { username if username.strip() else 'admin': password for username, password in users.items() }
        username = next(iter(users.keys()))
        password = pwdtools.hash_password(users[username])
        # Config file content
        config_content = textwrap.dedent(f'''\
            ### Config file of the Towalink WireGuard Frontend ###
            [general]
            # The WireGuard config file to read and write
            # wg_configfile = /etc/wireguard/wg_rw.conf
            wg_configfile = {wg_configfile}
    
            # The command to be executed when the WireGuard config has changed
            # on_change_command = 
            on_change_command = "sudo --non-interactive wg-quick down {wg_configfile}; sudo --non-interactive wg-quick up {wg_configfile}"
    
            # The interface the web server shall bind to
            # socket_host = 0.0.0.0
            socket_host = {socket_host}
    
            # The port the web server shall bind to
            # socket_port = 8080
            socket_port = {socket_port}
            
            # The system user to be used for the frontend
            # user = wgfrontend
            user = {user}
            
            [users]
            {username} = {password}
        ''')
        # Write to file system
        try:
            with open(self.filename, 'w') as config_file:
                config_file.write(config_content)
        except OSError as e:
            logger.error('Could not write config file [{0}], [{1}]'.format(self.filename, str(e)))

    @property
    def filename(self):
        """Return the name of the config file (incl. path)"""
        return config_filename

    @property
    def config(self):
        """Return the config dictionary"""
        if self._config is None:
            self.read_config()
        return self._config

    @property
    def users(self):
        """Return the users dictionary"""
        if self._users is None:
            self.read_config()
        return self._users

    @property
    def wg_configfile(self):
        """The filename incl. path of the config file for the WireGuard interface"""
        return self.config.get('wg_configfile', '/etc/wireguard/wg_rw.conf')

    @property
    def wg_interface(self):
        """The name of the WireGuard interface derived from the name of the WireGuard config file"""
        return os.path.basename(self.wg_configfile).rpartition('.')[0] # filename without extension

    @property
    def sslcertfile(self):
        """The filename incl. path for the server certificate"""
        return os.path.join(os.path.dirname(self.filename), 'server.pem')

    @property
    def sslkeyfile(self):
        """The filename incl. path for the server private key"""
        return os.path.join(os.path.dirname(self.filename), 'key.pem')

    @property
    def libdir(self):
        """The directory for the generated config files"""
        return '/var/lib/wgfrontend'

    @property
    def on_change_command(self):
        """The command to be executed on config changes"""
        cmd = self.config.get('on_change_command')
        if cmd is not None:
            if cmd[0] in ['"', '\'']:
                cmd = cmd[1:-1]
        return cmd

    @property
    def socket_host(self):
        """The interface to bind to"""
        return self.config.get('socket_host', '0.0.0.0')

    @property
    def socket_port(self):
        """The port to bind to"""
        return int(self.config.get('socket_port', 8080))

    @property
    def user(self):
        """The configured name for the wgfrontend system user"""
        return self.config.get('user', 'wgfrontend')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import cherrypy
import jinja2
import logging
import os
import random
import string
import subprocess

from . import pwdtools
from . import setupenv
from . import wgcfg


class WebApp():

    def __init__(self, cfg):
        """Instance initialization"""
        self.cfg = cfg
        self.jinja_env = jinja2.Environment(loader=jinja2.FileSystemLoader(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates')))
        self.wg = wgcfg.WGCfg(self.cfg.wg_configfile, self.cfg.libdir, self.on_change_func)

    @cherrypy.expose
    def index(self, action=None, id=None, description=None):
        if (action == 'delete') and id:
            peer, peerdata = self.wg.get_peer_byid(id)
            self.wg.delete_peer(peer)
        peers = self.wg.get_peers()
        tmpl = self.jinja_env.get_template('index.html')
        return tmpl.render(sessiondata=cherrypy.session, peers=peers)

    @cherrypy.expose
    def config(self, action=None, id=None, description=None):
        peerdata = None
        if (action == 'save') and id:
            peer, peerdata = self.wg.get_peer_byid(id)
            peerdata = self.wg.update_peer(peer, description)
        if (action == 'save') and not id:
            peer = self.wg.create_peer(description)
            peerdata = self.wg.get_peer(peer)
        if not peerdata:
            peer, peerdata = self.wg.get_peer_byid(id)
        tmpl = self.jinja_env.get_template('config.html')
        return tmpl.render(sessiondata=cherrypy.session, peerdata=peerdata)

    @cherrypy.expose
    def edit(self, action='edit', id=None, description=None):
        if id: # existing client
            peer, peerdata = self.wg.get_peer_byid(id)
            if description:
                peerdata = self.wg.update_peer(peer, description)
        else:
            if not description:
                description = 'My new client'
            if action == 'new': # default values for new client
                peerdata = { 'Description': description, 'Id': '' }
            else: # save changes
                raise ValueError()
        tmpl = self.jinja_env.get_template('edit.html')
        return tmpl.render(sessiondata=cherrypy.session, peerdata=peerdata)

    @cherrypy.expose
    def download(self, id):
        """Provide the WireGuard config for the client with the given identifier for download"""
        peer, peerdata = self.wg.get_peer_byid(id)
        config, peerdata = self.wg.get_peerconfig(peer)
        cherrypy.response.headers['Content-Disposition'] = f'attachment; filename=wg_{id}.conf'
        cherrypy.response.headers['Content-Type'] = 'text/plain' # 'application/x-download' 'application/octet-stream'
        return config.encode('utf-8')

    def check_username_and_password(self, username, password):
        """Check whether provided username and password are valid when authenticating"""
        if (username in self.cfg.users) and (pwdtools.verify_password(self.cfg.users[username], password)):
            return
        return 'invalid username/password'

    def login_screen(self, from_page='..', username='', error_msg='', **kwargs):
        """Shows a login form"""
        tmpl = self.jinja_env.get_template('login.html')
        return tmpl.render(from_page=from_page, username=username, error_msg=error_msg).encode('utf-8')

    @cherrypy.expose
    def logout(self):
        username = cherrypy.session['username']
        cherrypy.session.clear()
        cherrypy.response.headers['Cache-Control'] = 'no-cache, no-store, must-revalidate'
        cherrypy.response.headers['Pragma'] = 'no-cache'
        cherrypy.response.headers['Expires'] = '0'
        raise cherrypy.HTTPRedirect('/', 302)        
        return '"{0}" has been logged out'.format(username)

    def on_change_func(self):
        """React on config changes"""
        on_change_command = self.cfg.on_change_command
        if (on_change_command is not None) and (len(on_change_command) > 0):
            returncode = subprocess.call(on_change_command, shell=True)
            if returncode != 0:
                cherrypy.log('Error calling on_change_command', context='WEBAPP', severity=logging.ERROR, traceback=False)


def run_webapp(cfg):
    """Runs the CherryPy web application with the provided configuration data"""
    script_path = os.path.dirname(os.path.abspath(__file__))
    app = WebApp(cfg)
    # Configure the web application
    app_conf = {
      'global': {
         'environment' : 'production'
       },
       '/': {
            'tools.sessions.on': True,
            'tools.sessions.secure': True,
            'tools.sessions.httponly': True,
            'tools.staticdir.root': os.path.join(script_path, 'webroot'),
            'tools.session_auth.on': True,
            'tools.session_auth.login_screen': app.login_screen,
            'tools.session_auth.check_username_and_password': app.check_username_and_password,
            },
        '/configs': {
            'tools.staticdir.on': True,
            'tools.staticdir.root': None,
            'tools.staticdir.dir': cfg.libdir
        },
        '/static': {
            'tools.session_auth.on': False,
            'tools.staticdir.on': True,
            'tools.staticdir.dir': 'static'
        },
        '/favicon.ico':
        {
            'tools.session_auth.on': False,
            'tools.staticfile.on': True,
            'tools.staticfile.filename': os.path.join(script_path, 'webroot', 'static', 'favicon.ico')
        }
    }
    # Use SSL if certificate files exist
    if os.path.exists(cfg.sslcertfile) and os.path.exists(cfg.sslkeyfile):
        # Use ssl/tls if certificate files are present
        cherrypy.server.ssl_module = 'builtin'
        cherrypy.server.ssl_certificate = cfg.sslcertfile
        cherrypy.server.ssl_private_key = cfg.sslkeyfile
    # Define socket parameters
    cherrypy.config.update({'server.socket_host': cfg.socket_host,
                            'server.socket_port': cfg.socket_port,
                           })
    # Select environment
    cherrypy.config.update({'staging':
                             {
                               'environment' : 'production'
                             }
                           })
    # Start CherryPy
    cherrypy.tree.mount(app, config=app_conf)
    if setupenv.is_root():
        # Drop privileges
        uid, gid = setupenv.get_uid_gid(cfg.user, cfg.user)
        cherrypy.process.plugins.DropPrivileges(cherrypy.engine, umask=0o022, uid=uid, gid=gid).subscribe()
    cherrypy.engine.start()
    cherrypy.engine.signals.subscribe()
    cherrypy.engine.block()


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import getpass
import grp
import ipaddress
import os
import pwd
import string
import subprocess
import textwrap
import wgconfig
import wgconfig.wgexec as wgexec

from . import config
from . import exechelper
from . import setupenv_alpine


def is_root():
    """Returns whether this script is run with user id 0 (root)"""
    return os.getuid() == 0
    
def get_user():
    """Returns the effective user that executes this script"""
    return getpass.getuser()

def check_user(username):
    """Returns a boolean that indicates if the given user exists on the system"""
    try:
        pwd.getpwnam(username)
        return True
    except KeyError:
        return False

def create_user(username):
    """Create the given user on the system"""
    if os.path.exists('/usr/sbin/useradd'):
        os.system(f'useradd {username}')
    else:
        os.system(f'adduser {username} -D')
    
def ensure_user(username):
    """Ensures that the given user exists on the system"""
    if not check_user(username):
        create_user(username)

def check_wg():
    """Check whether the wg tool is present"""
    return os.path.isfile('/usr/bin/wg')

def check_wgquick():
    """Check whether the wg-quick tool is present"""
    return os.path.isfile('/usr/bin/wg-quick')

def touch_file(filename, perm=0o640):
    """Touch the given file with the provided permissions"""
    if not os.path.exists(os.path.dirname(filename)):
        os.makedirs(os.path.dirname(filename))
    with os.fdopen(os.open(filename, os.O_WRONLY | os.O_CREAT, perm), 'w') as handle:
        pass

def check_validcharacters(value, valid_chars=string.ascii_letters):
    """Checks whether the provided string only contains the listed characters"""
    invalid = [ k for k in value if k not in valid_chars ]
    if invalid:
        return False
    return True

def chown(username, path):
    """Change file/path ownership to given user"""
    uid = pwd.getpwnam(username).pw_uid
    os.chown(path, uid, -1)

def get_uid_gid(uid_name='nobody', gid_name='nogroup'):
    """Returns uid and gid for the given username and groupname"""
    uid = pwd.getpwnam(uid_name).pw_uid
    gid = grp.getgrnam(gid_name).gr_gid
    return uid, gid

def drop_privileges(uid_name='nobody', gid_name='nogroup'):
    """Drop privileges towards the given username and groupname"""
    if not is_root():
        raise ValueError('No privileges present to drop')
    uid, gid = get_uid_gid(uid_name, gid_name) 
    os.setgroups([]) # remove group privileges
    os.setgid(gid)
    os.setuid(uid)

def get_primary_interface():
    """Returns the name of the network interface having the default route"""
    exitcode, interface_name = subprocess.getstatusoutput("ip route | awk '/default/ { print $5 }'")
    if exitcode == 0:
        return interface_name
    else:
        return None

def get_primary_interface_addr4():
    """Returns the first IPv4 address of the network interface having the default route"""
    interface_name = get_primary_interface()
    if interface_name is None:
        return None
    exitcode, output = subprocess.getstatusoutput(f'ip addr show dev {interface_name}' + "| awk '/inet / { print $2 }'")
    if exitcode == 0:
        addr4 = output.partition('\n')[0] # get first line
        return addr4
    else:
        return None

def get_second_subnet():
    """Returns the second /28 subnet of the local network as well as the local network"""
    ip = get_primary_interface_addr4()
    net = ipaddress.ip_interface(ip).network
    nets = net.subnets(new_prefix=28)
    next(nets)
    return next(nets), net


class QueryUser():
    """Interact with the user"""

    def __init__(self):
        """Object initialization"""
        self._expert = None

    def input_yes_no(self, display_text, default='Yes', expert_question=False):
        """Queries the user for a yes or no answer"""
        if expert_question is not None:
            if expert_question and not self.expert:
                return default
        while True:
            answer = input(f'  {display_text} ')
            answer = answer.strip()
            if not answer:
                answer = default
            answer = answer.lower()
            if answer in ['1', 'y', 'yes']:
                return True
            if answer in ['0', 'n', 'no']:
                return False
            print('   Invalid answer. Please answer yes or no.') 

    def query_expert(self):
        """Queries the user on whether he wants expert configuration"""
        expert = self.input_yes_no('Do you want to use expert configuration? [No]:', default='No', expert_question=None)
        return expert

    @property
    def expert(self):
        """Query user on first run on whether expert configuration is desired and remember result"""
        if self._expert is None:
            self._expert = self.query_expert()
        return self._expert

    def get_input(self, display_text):
        """Queries the user for input"""
        userdata = input(f'  {display_text} ')
        userdata = userdata.strip()
        return userdata

    def get_and_validate_input(self, display_text, default=None, check_function=None, expert_question=False):
        """Queries the user for input and validates it"""
        if expert_question and not self.expert:
            return default
        ok = False
        while not ok:
            userdata = self.get_input(display_text)
            if not userdata:
                userdata = default
            if check_function is None:
                ok = True
            else:
                userdata = check_function(userdata)
                if userdata:
                    ok = True
        if userdata:
            return userdata
        else:
            return default

    def get_wg_configfile(self):
        """Query the user for the path of the WireGuard config file"""
        return self.get_and_validate_input('Please specify the WireGuard config file to be used [/etc/wireguard/wg_rw.conf]:', default='/etc/wireguard/wg_rw.conf', expert_question=True)

    def get_system_user(self):
        """Query the user for the system user for the web frontend"""
        return self.get_and_validate_input('Please specify the system user for the web frontend [wgfrontend]:', default='wgfrontend', expert_question=True)

    def get_socket_host(self):
        """Query the user for the listening interface for the web server"""

        def check(userdata):
            if check_validcharacters(userdata, string.hexdigits + '.:'):
                return userdata
            print('    Invalid characters entered. Please enter anew.')
            return None

        return self.get_and_validate_input('Please specify the listening interface for the web server [0.0.0.0]:', default='0.0.0.0', check_function=check, expert_question=True)

    def get_socket_port(self):
        """Query the user for the listening port for the web server"""

        def check(userdata):
            if (not userdata) or userdata.isdigit():
                return userdata
            print('    You need to provide a port number. Please enter anew.')
            return None

        return self.get_and_validate_input('Please specify the listening port for the web server [8080]:', default='8080', check_function=check, expert_question=True)

    def get_frontend_username(self):
        """Query the user for the username for the web frontend user"""

        def check(userdata):
            if check_validcharacters(userdata, string.ascii_letters + '_'):
                return userdata
            print('    Username must only contain letters and underscores. Please enter anew.')
            return None

        return self.get_and_validate_input('Please specify the username for your web frontend user [admin]:', default='admin', check_function=check, expert_question=False)

    def get_frontend_password(self):
        """Query the user for the password for the web frontend user"""

        def check(userdata):
            if len(userdata) >= 8:
                return userdata
            print('    Password must have at least eight characters. Please enter anew.')
            return None

        return self.get_and_validate_input('Please specify the password for your web frontend user:', check_function=check, expert_question=False)

    def get_wg_listenport(self):
        """Query the user for the listen port of the WireGuard interface"""

        def check(userdata):
            if str(userdata).isdigit():
                return int(userdata)
            print('    You need to provide a numeric port number. Please enter anew.')
            return None

        return self.get_and_validate_input('Please specify the listen port of the WireGuard interface [51820]:', default=51820, check_function=check, expert_question=False)

    def get_endpoint(self):
        """Query the user for the endpoint hostname (and optionally port) to reach the WireGuard server"""

        def check(userdata):
            if len(userdata) > 0:
                return userdata
            print('    You need to enter an endpoint hostname. Please enter anew.')
            return None

        print('  You need to specify the endpoint hostname (and optionally port) to reach your WireGuard server.')
        print('  In a home environment, this is usually a DynDNS name denoting your Internet router.')
        return self.get_and_validate_input('Please specify the endpoint hostname (and optionally port) to reach your WireGuard server:', default='', check_function=check, expert_question=False)

    def get_wg_address(self, default='192.168.0.17/28'):
        """Query the user for the IP address of the WireGuard interface incl. prefix length"""

        def check(userdata):
            try:
                userdata = ipaddress.ip_interface(userdata)
                return userdata
            except e:
                print('  Exception: {text}'.format(text=str(e)))
            return None

        return self.get_and_validate_input(f'Please specify the IP address of the WireGuard interface incl. prefix length [{default}]:', default=default, check_function=check, expert_question=False)

    def get_wg_networks(self):
        """Query the user for the network ranges that the clients shall route to the WireGuard server"""
        return self.get_and_validate_input('Please specify the network ranges that the clients shall route to the WireGuard server [192.168.0.0/16]:', default='192.168.0.0/16', expert_question=False)


def setup_environment():
    """Environment setup assistant"""
    cfg = config.Configuration()
    if is_root():
        qu = QueryUser()
        print('Welcome to Towalink WireGuard Frontend')
        print('======================================')
        print('You are executing "wgfrontend" as root user. We\'ll now make sure that everything is properly installed.')
        if check_wg():
            print('Wireguard (wg) is available. Ok.')
        else:
            print('Wireguard (wg) is not available. FAIL.')
        if check_wg():
            print('Wireguard (wg-quick) is available. Ok.')
        else:
            print('Wireguard (wg-quick) is not available. FAIL.')
        if cfg.exists():
            print(f'Config file {cfg.filename} already exists. Ok.')
        else:
            print(f'Config file {cfg.filename} does not yet exist. Let\'s create one...')
            print('  Press enter to select defaults.')
            wg_configfile = qu.get_wg_configfile()
            user = qu.get_system_user()
            socket_host = qu.get_socket_host() # listening interface for the web server
            socket_port = qu.get_socket_port() # listening port for the web server
            username = qu.get_frontend_username()
            password = qu.get_frontend_password()
            touch_file(cfg.filename, perm=0o640) # create without world read permissions
            cfg.write_config(wg_configfile=wg_configfile, socket_host=socket_host, socket_port=socket_port, user=user, users={username: password})
            print('  Config file written. Ok.')
        print(f'Ensuring that system user "{cfg.user}" exists.')
        ensure_user(cfg.user)
        print(f'Ensuring ownership of config file {cfg.filename}.')
        chown(cfg.user, cfg.filename)
        if os.path.exists(cfg.libdir):
            print(f'Directory {cfg.libdir} already exists. Ok.')
        else:
            print(f'Directory {cfg.libdir} does not yet exist. Let\'s create it...')
            os.makedirs(cfg.libdir, mode=0o750, exist_ok=True)
            print('  Directory created. Ok.')
        print(f'Ensuring ownership of directory {cfg.libdir}.')
        chown(cfg.user, cfg.libdir)
        if os.path.exists(cfg.wg_configfile):
            print(f'WireGuard config file {cfg.wg_configfile} already exists. Ok.')
        else:
            network_subrange, network_local = get_second_subnet()
            ip_wgif = network_subrange[1]
            print(f'WireGuard config file {cfg.wg_configfile} does not yet exist. Let\'s create one...')
            print('  For documentation on possible setups, please refer to')
            print('  https://github.com/towalink/wgfrontend/tree/main/doc/network-integration')
            print('  Automated configuration is only supported for the ProxyARP setup.')
            print('  For this, choose an unused subrange of your local network for WireGuard.')
            print(f'  For your local network {network_local.exploded}, you could e.g. use the subrange')
            print(f'  {network_subrange.exploded} with {ip_wgif}/{network_subrange.prefixlen} as the WireGuard interface address')
            print(f'  and the other addresses for clients. For this, the addresses from {network_subrange.network_address}')
            print(f'  to {network_subrange.broadcast_address} must not be in use in your network.')
            print(f'  Press enter to select defaults.')
            wg_listenport = qu.get_wg_listenport()
            endpoint = qu.get_endpoint()
            wg_address_obj = qu.get_wg_address(default=ip_wgif.exploded + '/' + str(network_subrange.prefixlen))
            wg_networks = qu.get_wg_networks()
            # Check for ProxyARP setup
            proxy_arp_interface = None
            eth_address = get_primary_interface_addr4()
            if (eth_address is not None) and (len(eth_address) > 0):
                eth_address_obj = ipaddress.ip_interface(eth_address)
                if wg_address_obj.network.subnet_of(eth_address_obj.network):
                    interface_name = get_primary_interface()
                    print('  Setup for ProxyARP detected.')
                    if qu.input_yes_no(f'7e) Do you want to configure ProxyARP on interface {interface_name} when bringing up the WireGuard interface? [Yes]: ', expert_question=True):
                        proxy_arp_interface = interface_name
                else:
                    print('  Please configure your network setup based on the documentation referenced above.')
            # Write WireGuard config file
            wc = wgconfig.WGConfig(cfg.wg_configfile)
            wc.initialize_file('# This file has been created and is managed by wgfrontend. Only change manually if you know what you\'re doing.')
            if not ':' in endpoint:
                endpoint += ':51820'
            wc.add_attr(None, 'ListenPort', wg_listenport, '# Endpoint = ' + endpoint, append_as_line=True)
            wc.add_attr(None, 'PrivateKey', wgexec.generate_privatekey())
            wc.add_attr(None, 'Address', wg_address_obj.compressed, '# Networks = ' + wg_networks, append_as_line=True)
            if proxy_arp_interface is not None:
                wc.add_attr(None, 'PostUp', f'sysctl -w net.ipv4.conf.{proxy_arp_interface}.proxy_arp=1', append_as_line=True)
            wc.write_file()
            print('  Config file written. Ok.')
            eh = exechelper.ExecHelper()
            if qu.input_yes_no(f'Would you like to enable IP Forwarding so that this device can act as a router? [Yes]:', expert_question=True):
                eh.execute('sysctl -w net.ipv4.ip_forward=1', suppressoutput=True, suppresserrors=False)
                eh.execute('sysctl -w net.ipv6.conf.all.forwarding=1', suppressoutput=True, suppresserrors=False)
                ipforwarding_content = textwrap.dedent(f'''\
                    net.ipv4.ip_forward = 1
                    net.ipv6.conf.all.forwarding = 1
                ''')
                if os.path.isdir('/etc/sysctl.d'):
                    with open('/etc/sysctl.d/99-wgfrontend-forwarding.conf', 'w') as ipforwarding_file:
                        ipforwarding_file.write(ipforwarding_content)
                else:
                    print('  Sorry, "/etc/sysctl.d" does not exist so that we could not install a config file there.')
            if qu.input_yes_no(f'Would you like to allow the system user of the web frontend to reload WireGuard on config changes (using sudo)? [Yes]:'):
                sudoers_content = textwrap.dedent(f'''\
                    {cfg.user}  ALL=(root) NOPASSWD: /etc/init.d/wgfrontend_interface start, /etc/init.d/wgfrontend_interface stop, /etc/init.d/wgfrontend_interface restart
                    {cfg.user}  ALL=(root) NOPASSWD: /usr/bin/wg-quick down {cfg.wg_configfile}, /usr/bin/wg-quick up {cfg.wg_configfile}
                ''')    
                if os.path.isdir('/etc/sudoers.d'):
                    with open('/etc/sudoers.d/wgfrontend', 'w') as sudoers_file:
                        sudoers_file.write(sudoers_content)
                else:
                    print('  Sorry, "/etc/sudoers.d" does not exist so that sudo could not be configured. Maybe "sudo" is not installed.')
            if qu.input_yes_no(f'Would you like to activate the WireGuard interface "{cfg.wg_interface}" now? [Yes]:', expert_question=True):
                eh.run_wgquick('up', cfg.wg_interface)
            if qu.input_yes_no(f'Would you like to activate the WireGuard interface "{cfg.wg_interface}" on boot? [Yes]:', expert_question=True):
                if eh.os_id == 'alpine':
                    setupenv_alpine.start_wginterface_onboot()
                else:
                    eh.enable_service(f'wg-quick@{cfg.wg_interface}')
            if qu.input_yes_no(f'Would you like to start wgfrontend on boot? [Yes]:', expert_question=True):
                if eh.os_id == 'alpine':
                    setupenv_alpine.start_wgfrontend_onboot()
                else:
                    # Create and install systemd service
                    systemd_content = textwrap.dedent(f'''\
                        [Unit]
                        Description=wgfrontend service
                        After=network-online.target
                        Wants=network-online.target
                        
                        [Service]
                        Type=simple
                        ExecStart=wgfrontend
                        User={cfg.user}
                        Group={cfg.user}
                        StandardOutput=append:/var/log/wgfrontend.log
                        StandardError=inherit
                        
                        [Install]
                        WantedBy=multi-user.target
                    ''')    
                    if os.path.isdir('/etc/systemd/system'):
                        with open('/etc/systemd/system/wgfrontend.service', 'w') as systemd_file:
                            systemd_file.write(systemd_content)  
                        eh.execute('systemctl daemon-reload', suppressoutput=True, suppresserrors=True)
                        eh.enable_service('wgfrontend')
                    else:
                        print('  Sorry, "/etc/systemd/system" does not exist so that the service file could not be installed.')
        print(f'Ensuring list permission of WireGuard config directory {os.path.dirname(cfg.wg_configfile)}.')
        os.chmod(os.path.dirname(cfg.wg_configfile), 0o711)
        print(f'Ensuring ownership of WireGuard config file {cfg.wg_configfile}.')
        chown(cfg.user, cfg.wg_configfile)
        if os.path.exists(cfg.sslcertfile):
            print(f'Ensuring ownership of server certificate file {cfg.sslcertfile}.')
            chown(cfg.user, cfg.sslcertfile)
        if os.path.exists(cfg.sslkeyfile):
            print(f'Ensuring ownership of server private key file {cfg.sslkeyfile}.')
            chown(cfg.user, cfg.sslkeyfile)    
        print()
        print('Attempting to start web frontend...')
    return cfg


if __name__ == '__main__':
    setup_environment()



# -*- coding: utf-8 -*-

import os
import textwrap


def enable_startscript(service):
    """Start the given service on boot"""
    ret = os.system('rc-update add {service}'.format(service=service))
    return ret

def get_startupscript_wgfrontend():
    """Get a startup script for wgfrontend"""
    template = r'''
        #!/sbin/openrc-run
        
        depend() {
                need net
        }
        
        name=$RC_SVCNAME
        command="wgfrontend"
        command_args=""
        pidfile="/run/$RC_SVCNAME.pid"
        command_background="yes"
        stopsig="SIGTERM"
        start_stop_daemon_args="--stdout /var/log/wgfrontend.log --stderr /var/log/wgfrontend.err"
        #Let wgfrontend drop privileges so that we keep permission to write to the log files
        #command_user="wgfrontend:wgfrontend"
        '''
    template = textwrap.dedent(template).lstrip()
    return template

def get_startupscript_wginterface(interface_name='wg_rw'):
    """Get a startup script for the WireGuard interface used by wgfrontend"""
    template = r'''
        #!/sbin/openrc-run
        
        depend() {{
                need net
        }}
        
        start() {{
                wg-quick up {interface_name}
        }}
        
        stop() {{
                wg-quick down {interface_name}
        }}
        
        name=$RC_SVCNAME
        '''
    template = textwrap.dedent(template).lstrip()
    return template.format(interface_name=interface_name)

def write_startupscript_wgfrontend():
    """Write a startup script for wgfrontend to /etc/init.d"""
    filename = '/etc/init.d/wgfrontend'
    with open(filename, 'w') as f:
        f.write(get_startupscript_wgfrontend())
    os.chmod(filename, 0o700)

def write_startupscript_wginterface():
    """Write a startup script for the WireGuard interface of wgfrontend to /etc/init.d"""
    filename = '/etc/init.d/wgfrontend_interface'
    with open(filename, 'w') as f:
        f.write(get_startupscript_wginterface())
    os.chmod(filename, 0o700)

def start_wgfrontend_onboot():
    """Enable the start script of wgfrontend"""
    write_startupscript_wgfrontend()
    enable_startscript('wgfrontend')
    
def start_wginterface_onboot():
    """Enable the start script for the WireGuard interface of wgfrontend"""
    write_startupscript_wginterface()
    enable_startscript('wgfrontend_interface') 
